/* **********************************************************************
 *  JSW:O ALERT
 *  Version: 1.4.3 (04-Sep-2011)
 *  Author: Kris King (aka Fragile)
 *
 *  See licence.txt and readme.txt for more information
 *
 *  Send bug reports, suggestions, and comments to:
 *  entrancetohades@distant-angel.co.uk
 *
 *  ALERT.JS
 *  Populate and display message alerts
 * ********************************************************************** */

var JSWOA_alert = {

	// Pref defaults
	prfOpenTime: 3000, // Time alert remains open (default 3 sec.)
	prfSlideDel: 10, // Delay (in ms) between slide increments
	prfPlaySnd: true, // Whether or not to play the alert sound

	// Set some variables
	finalHeight: 50,
	slideInc: 2,
	slideTimer: null, // Instance of nsITimer for slide
	alertListener: null,
	alertCookie: "",


	/* ANIMATE()
	 * Slide animate the alert
	 */
	animate: function() {
		try {
			// Play sound
			if(window.outerHeight == 1 && JSWOA_alert.prfPlaySnd)
				JSWOA_alert.playSound();
			// Slide window up
			if(window.outerHeight < JSWOA_alert.finalHeight) {
				window.screenY -= JSWOA_alert.slideInc;
				window.outerHeight += JSWOA_alert.slideInc;
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_alert.animate(); } };
				JSWOA_alert.slideTimer.init(timeEvt, JSWOA_alert.prfSlideDel, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
			} else {
				// Done sliding, set timeout for wait to close
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_alert.close(); } };
				JSWOA_alert.slideTimer.init(timeEvt, JSWOA_alert.prfOpenTime, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
			}
		} catch(e) { alert("JSWOA_alert.animate()\n\n" + e); }
	},


	/* CLICK()
	 * Respond to an alert popup click
	 */
	click: function(event, ip, port) {
		try {
			if(event.button == 0) {
				// Launch specific server (JSW:O v3.x only)
				if(JSWOA_core.getPref("char", "jswoversion") == "3.x" && JSWOA_core.getPref("bool", "jswolaunchwilly") && ip) {
					var winEnum = JSWOA_core.getWindowEnumerator();
					if(typeof winEnum != "undefined" && winEnum.hasMoreElements())
						winEnum.getNext().JSWOA_ui.launchJSWO(ip, port);
				} else {
					// Close popup (if preference set)
					var prfLeftClick = JSWOA_core.getPref("bool", "leftclickclose");
					if(prfLeftClick) {
						JSWOA_alert.slideTimer.cancel();
						JSWOA_alert.close();
					}
				}
			}
		} catch(e) { alert("JSWOA_alert.click()\n\n" + e); }
	},


	/* CLOSE()
	 * Close alert popup
	 */
	close: function() {
		try {
			// Slide window down
			if(window.outerHeight > 1 && JSWOA_alert.prfSlideDel != 0) {
				window.screenY += JSWOA_alert.slideInc;
				window.outerHeight -= JSWOA_alert.slideInc;
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_alert.close(); } };
				JSWOA_alert.slideTimer.init(timeEvt, JSWOA_alert.prfSlideDel, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
			} else {
				// Done sliding, close window
				if(JSWOA_alert.alertListener)
					JSWOA_alert.alertListener.observe(null, "alertfinished", JSWOA_alert.alertCookie);
				window.close();
			}
		} catch(e) { alert("JSWOA_alert.close()\n\n" + e); }
	},


	/* LOAD()
	 * Called when alert window opens (main entry point)
	 */
	load: function() {
		try {
			// Get timer service
			JSWOA_alert.slideTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
			// Get alert preferences
			JSWOA_alert.prfOpenTime = (JSWOA_core.getPref("int", "alertopentime") * 1000);
			JSWOA_alert.prfSlideDel = JSWOA_core.getPref("int", "alertslidedelay");
			JSWOA_alert.prfPlaySnd = JSWOA_core.getPref("bool", "alertplaysound");
			// Resize popup
			sizeToContent();
			JSWOA_alert.finalHeight = window.outerHeight;
			window.outerHeight = 1;
			// Offset alert by 10 pixels from the far right edge of the screen
			window.moveTo((screen.availLeft + screen.availWidth - window.outerWidth) - 10, screen.availTop + screen.availHeight - window.outerHeight);
			if(JSWOA_alert.prfSlideDel == 0) {
				// Instant display
				window.outerHeight = JSWOA_alert.finalHeight;
				window.screenY -= JSWOA_alert.finalHeight;
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_alert.close(); } };
				JSWOA_alert.slideTimer.init(timeEvt, JSWOA_alert.prfOpenTime, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
			} else {
				// Sliding display
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_alert.animate(); } };
				JSWOA_alert.slideTimer.init(timeEvt, JSWOA_alert.prfSlideDel, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
			}
			JSWOA_alert.populate();
		} catch(e) { alert("JSWOA_alert.load()\n\n" + e); }
	},


	/* PLAYSOUND()
	 * Play alert sound
	 */
	playSound: function() {
		try {
			// Get sound service and play system notification sound
			var sndSvc = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
			sndSvc.playSystemSound("SystemNotification");
		} catch(e) { alert("JSWOA_alert.playSound()\n\n" + e); }
	},


	/* POPULATE()
	 * Fill alert with details of message
	 */
	populate: function() {
		try {
			// Set some variables
			var msgArr = window.arguments[0];
			var tVBox = document.getElementById("jswoa_alert_block");
			var tHBox;
			var tHBoxIn;
			var tTBox;

			// Get version/flag prefs
			var prfJSWOVersion = JSWOA_core.getPref("char", "jswoversion");
			var prfShowFlag = JSWOA_core.getPref("bool", "alertflag");

			// Set background/title colours, and gradient
			tHBox = tVBox.firstChild;
			tHBox.setAttribute("style", ("background-color: " + JSWOA_core.getPref("char", "alertback") + ";" + (!JSWOA_core.getPref("bool", "alertgradient") ? " background-image: url(none);" : "")));
			tHBox.firstChild.nextSibling.firstChild.nextSibling.firstChild.setAttribute("style", "color: " + JSWOA_core.getPref("char", "alerttitle") + ";");

			// Set description/type text colour
			var prfText = JSWOA_core.getPref("char", "alerttext");
			tHBox.firstChild.nextSibling.firstChild.nextSibling.firstChild.nextSibling.setAttribute("style", "color: " + prfText + ";");
			tHBox.firstChild.nextSibling.firstChild.nextSibling.firstChild.nextSibling.nextSibling.setAttribute("style", "color: " + prfText + ";");

			// Loop through all messages
			for(i in msgArr) {

				// First message uses existing hBox, others create a clone of it
				tHBox = (i == 0) ? tHBox : tVBox.firstChild.cloneNode(true);
				tHBoxIn = tHBox.firstChild.nextSibling;

				// Set onClick event handlers for main box and Willy (JSW:O v3.x only)
				tHBoxIn.addEventListener("click", function (event) { JSWOA_alert.click(event); }, false);
				if(prfJSWOVersion == "3.x" && JSWOA_core.getPref("bool", "jswolaunchwilly") && msgArr[i]["IP"] != "")
					tHBox.firstChild.setAttribute("style", "cursor: pointer;");
				tHBox.firstChild.addEventListener("click", function (event) { if(prfJSWOVersion == "3.x" && msgArr[i]["IP"] != "") JSWOA_alert.click(event, msgArr[i]["IP"], msgArr[i]["Port"]); else JSWOA_alert.click(event); }, false);
				tHBoxIn.firstChild.setAttribute("hidden", true);

				// Set country flag (JSW:O v3.x only)
				if(prfJSWOVersion == "3.x" && prfShowFlag && msgArr[i]["Cnty"] != "") {
					var coPos = JSWOA_core.getFlagPos(msgArr[i]["Cnty"]);
					if(coPos != "") {
						tHBoxIn.firstChild.setAttribute("hidden", false);
						tHBoxIn.firstChild.setAttribute("style", "-moz-image-region: rect(" + coPos + ");");
					}
				}

				// Set alert text labels
				tTBox = tHBoxIn.firstChild.nextSibling;
				tTBox.firstChild.setAttribute("value", msgArr[i]["Msg"]);
				tTBox.firstChild.nextSibling.setAttribute("value", msgArr[i]["Desc"]);
				tTBox.firstChild.nextSibling.nextSibling.setAttribute("value", msgArr[i]["Type"]);

				// If game is not passworded, don't display lock icon
				tTBox.nextSibling.firstChild.setAttribute("hidden", (msgArr[i]["Pass"] == "N") ? true : false);

				// Append new box (if not the first message)
				if(i != 0)
					tVBox.appendChild(tHBox);
			}
		} catch(e) { alert("JSWOA_alert.populate()\n\n" + e); }
	}

};
window.addEventListener("load", function() { JSWOA_alert.load(); }, false);
