/* **********************************************************************
 *  JSW:O ALERT
 *  Version: 1.4.3 (04-Sep-2011)
 *  Author: Kris King (aka Fragile)
 *
 *  See licence.txt and readme.txt for more information
 *
 *  Send bug reports, suggestions, and comments to:
 *  entrancetohades@distant-angel.co.uk
 *
 *  PREFERENCES.JS
 *  Handle the display, modification, validation, and submission of user
 *  preferences for JSW:O Alert
 * ********************************************************************** */

var JSWOA_pref = {

	// If these prefs change, we should re-contact the server
	trgPrfIgnorePwd: null,
	trgPrfFetchFreq: null,
	trgPrfJSWOVer: null,


	/* BROWSE()
	 * Browse for JSW executable file
	 */
	browse: function(txtFilebox) {
		try {
			// Set file extension filter
			var fpkMgr = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
			if(JSWOA_core.osPlatform == "winnt")
				fpkMgr.appendFilter(JSWOA_core.strBun.getString("jsw.prf.filesexe"),"*.exe");
			else if(JSWOA_core.osPlatform == "darwin")
				fpkMgr.appendFilter(JSWOA_core.strBun.getString("jsw.prf.filesapp"),"*.app;*.appl");
			// Open dialog to select a file
			fpkMgr.init(window, JSWOA_core.strBun.getString("jsw.prf.selectfolder"), fpkMgr.modeOpenFile);
			var tmpRet = fpkMgr.show();
			// User selected OK, insert into text box
			if(tmpRet == fpkMgr.returnOK && fpkMgr.file.path != '')
				document.getElementById(txtFilebox).value = fpkMgr.file.path;
		} catch(e) { alert("JSWOA_pref.browse()\n\n" + e); }
	},


	/* COMMANDEL()
	 * Enable/disable textboxes based on checkbox selection
	 */
	commandEL: function(event) {
		try {
			if(event.target.localName == "checkbox") {
				if(event.target.getAttribute("prefswitch") != "") {
					// Switch specified element on/off
					var swArr = event.target.getAttribute("prefswitch").split(",");
					// Loop through all link elements
					for(var i in swArr)
						document.getElementById(swArr[i]).disabled = !event.target.checked;
					// Disable elements on the "no enable" list
					var swNoArr = event.target.getAttribute("prefswitchnoenable").split(",");
					for(i in swNoArr)
						if(swNoArr[i])
							document.getElementById(swNoArr[i]).disabled = true;
				}
			}
			// Check whether we need to enable/disable a button
			JSWOA_pref.setButton(event.target.getAttribute("prefbutton"));
		} catch(e) { alert("JSWOA_pref.commandEL()\n\n" + e); }
	},


	/* INIT()
	 * Called when prefs window opens (main entry point)
	 */
	init: function() {
		try {
			// Set up event listeners
			window.addEventListener("command", JSWOA_pref.commandEL, false);
			window.addEventListener("keypress", JSWOA_pref.keyPressEL, false);
			window.addEventListener("keyup", JSWOA_pref.keyUpEL, false);
			window.addEventListener("select", JSWOA_pref.selectEL, false);

			// Set version number in the about tab
			document.getElementById("jswoa_pref_bannerhead").setAttribute("value", ("JSW:O Alert v" + JSWOA_core.getPref("char", "version")));

			// Get all elements with a prefname attribute
			var elArr = document.getElementsByAttribute("prefname", "*");
			for(var i in elArr) {
				try {
					// Get the preftype for this element
					var elPVal = JSWOA_core.getPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"));
					switch(elArr[i].localName) {
						// Set its default value, based on type
						case "textbox":
						case "radiogroup":
							elArr[i].value = elPVal;
							break;
						case "checkbox":
							elArr[i].checked = elPVal;
							break;
						case "colorpicker":
							elArr[i].color = elPVal;
							break;
						case "menulist":
							JSWOA_pref.menuSelect(elArr[i], elPVal);
							break;
						case "tree":
							JSWOA_pref.treeSet(elArr[i], JSWOA_pref.treeJL(elArr[i], elPVal, true));
							break;
						default:
							break;
					}
				} catch(e) {}
			}

			// Enable/disable JSW:O version-specific prefs
			JSWOA_pref.setVersionPrefs(JSWOA_core.getPref("char", "jswoversion"));

			// Ensure elements linked by checkboxes are correctly enabled/disabled
			var chkArr = document.getElementsByAttribute("prefswitch", "*");
			for(i in chkArr) {
				try {
					var swArr = chkArr[i].getAttribute("prefswitch").split(",");
					// Loop through link elements
					for(var j in swArr) {
						var tSwitch = document.getElementById(swArr[j]);
						if(chkArr[i].localName == "checkbox") {
							// Disable element if checkbox is off
							if(!chkArr[i].checked)
								tSwitch.disabled = true;
							else {
								// Switch off checkbox if element is a textbox with a zero or empty value
								if(tSwitch.localname == "textbox" && tSwitch.value == "0") {
									chkArr[i].checked = false;
									tSwitch.disabled = true;
								} else {
									// Switch off elements on the "no enable" list
									var swNoArr = chkArr[i].getAttribute("prefswitchnoenable").split(",");
									for(var k in swNoArr)
										document.getElementById(swNoArr[k]).disabled = true;
								}
							}
						}
					}
				} catch (e) {}
			}

			// Get trigger prefs
			JSWOA_pref.trgPrfIgnorePwd = JSWOA_core.getPref("bool", "ignorepwd");
			JSWOA_pref.trgPrfFetchFreq = JSWOA_core.getPref("int", "fetchfreq");
			JSWOA_pref.trgPrfJSWOVer = JSWOA_core.getPref("char", "jswoversion");
		} catch(e) { alert("JSWOA_pref.init()\n\n" + e); }
	},


	/* KEYPRESSEL()
	 * Restrict textbox input to numeric characters
	 */
	keyPressEL: function(event) {
		try {
			// Only for textboxes with a preftype of "int"
			if(event.target.localName == "textbox") {
				if(event.target.getAttribute("preftype") == "int") {
					event.target.oldvalue = event.target.value;
					if(event.charCode) {
						// Only allow a numeric keypress
						var key = String.fromCharCode(event.charCode).match(/\d/g);
						if(!key) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		} catch(e) { alert("JSWOA_pref.keyPressEL()\n\n" + e); }
	},


	/* KEYUPEL()
	 * Enable/disable linked button
	 */
	keyUpEL: function(event) {
		try {
			// Check whether we need to enable/disable a button
			if(event.target.localName == "textbox")
				JSWOA_pref.setButton(event.target.getAttribute("prefbutton"));
		} catch(e) { alert("JSWOA_pref.keyUpEL()\n\n" + e); }
	},


	/* MENUSELECT()
	 * Set the selected item in a dropdown list
	 */
	menuSelect: function(menu, val) {
		try {
			var mItems = menu.menupopup.childNodes;
			for(var i in mItems) {
				if(mItems[i].value == val) {
					menu.selectedIndex = i;
					break;
				}
			}
		} catch(e) { alert("JSWOA_pref.menuSelect()\n\n" + e); }
	},


	/* OPENLINK()
	 * Open a link in the browser
	 */
	openLink: function(url) {
		try {
			if(JSWOA_core.appCode == "tb") {
				// Thunderbird: launch URL in external browser
				var messenger = Components.classes['@mozilla.org/messenger;1'].createInstance();
				messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
				messenger.launchExternalURL(url);
			} else {
				// Firefox/SeaMonkey: get most recent window
				var winMgr = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService();
				var winMgrInt = winMgr.QueryInterface(Components.interfaces.nsIWindowMediator);
				var winRec = winMgrInt.getMostRecentWindow(JSWOA_core.appWinType);
				if(!winRec)
					// Open link in a new window (if no recent window exists)
					winRec = window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
				else {
					// Open new tab in the current window
					var cnt = winRec.document.getElementById("content");
					cnt.selectedTab = cnt.addTab(url);
				}
			}
		} catch(e) { alert("JSWOA_pref.openLink()\n\n" + e); }
	},


	/* SAVE()
	 * Validate and save prefs
	 */
	save: function() {
		try {
			// Get all elements with a prefname attribute
			var elArr = document.getElementsByAttribute("prefname", "*");
			var chkList = "";
			for(var i in elArr) {
				try {
					// Get the preftype for this element
					var pPref = JSWOA_core.getPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"));
					switch(elArr[i].localName) {
						// Save its value to preferences, based on type
						case "textbox":
						case "radiogroup":
							if(elArr[i].localName == "textbox" && elArr[i].getAttribute("preftype") == "int") {
								try {
									// Ensure value is within a specified range
									var pMin = Number(elArr[i].getAttribute("prefmin"));
									var pMax = Number(elArr[i].getAttribute("prefmax"));
									var pNewVal = (elArr[i].value == "") ? 0 : elArr[i].value;
									elArr[i].value = (pNewVal < pMin) ? pMin : (pNewVal > pMax) ? pMax : pNewVal;
									if(pNewVal == 0) {
										// Add checkboxes dependent on this pref to the list
										var pSw = elArr[i].getAttribute("prefswitch");
										if(pSw != "")
											chkList = (chkList + ((chkList != "") ? "," : "") + pSw);
									}
								} catch (e) {}
							}
							JSWOA_core.setPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"), elArr[i].value);
							break;
						case "checkbox":
							JSWOA_core.setPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"), elArr[i].checked);
							break;
						case "colorpicker":
							JSWOA_core.setPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"), elArr[i].color);
							break;
						case "menulist":
							JSWOA_core.setPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"), elArr[i].menupopup.childNodes[elArr[i].selectedIndex].value);
							break;
						case "tree":
							var jlStr = JSWOA_pref.treeJL(elArr[i], JSWOA_pref.treeGet(elArr[i]), false);
							JSWOA_core.setPref(elArr[i].getAttribute("preftype"), elArr[i].getAttribute("prefname"), jlStr);
							if(jlStr.length == 0) {
								// Add checkboxes dependent on this pref to the list
								var pSw = elArr[i].getAttribute("prefswitch");
								if(pSw != "")
									chkList = (chkList + ((chkList != "") ? "," : "") + pSw);
							}
							break;
						default:
							break;
					}
				} catch(e) { }
			}

			// Process list of dependent checkboxes
			if(chkList != "") {
				var chkArr = chkList.split(",");
				for(i in chkArr) {
					// Set this checkbox
					var chkEl = document.getElementById(chkArr[i]);
					JSWOA_core.setPref("bool", chkEl.getAttribute("prefname"), false);
				}
			}

			// If any of the trigger prefs have changed values, contact server again
			var doEmptyArr = (JSWOA_pref.trgPrfJSWOVer != document.getElementById("jswoa_pref_jswoversion").selectedItem.value) ? true : false;
			var doContact = (JSWOA_pref.trgPrfIgnorePwd != document.getElementById("jswoa_pref_ignorepwd").checked || JSWOA_pref.trgPrfFetchFreq != document.getElementById("jswoa_pref_fetchfreq").value || doEmptyArr);

			// Tell main window we've saved prefs
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				var oldWin = winEnum.getNext();
				// Empty old server list and reset status bar text if we've switched versions
				if(doEmptyArr) {
					oldWin.JSWOA_ui.lastSvrArr = null;
					oldWin.JSWOA_ui.lastSBText = JSWOA_core.formatStatus(oldWin.JSWOA_ui.netErr ? -1 : 0);
				}
				oldWin.JSWOA_ui.afterSavePref(doContact);
			}
			return true;
		} catch(e) { alert("JSWOA_pref.save()\n\n" + e); }
	},


	/* SELECTEL()
	 * Enable/disable linked button
	 */
	selectEL: function(event) {
		try {
			// Check whether we need to enable/disable a button
			if(event.target.localName == "tree")
				JSWOA_pref.setButton(event.target.getAttribute("prefbutton"));
		} catch(e) { alert("JSWOA_pref.selectEL()\n\n" + e); }
	},


	/* SETBUTTON()
	 * Enable/disable a button based on values of linked elements
	 */
	setButton: function(btn) {
		try {
			// Find elements this button depends on
			if(btn != "") {
				var depArr = document.getElementsByAttribute("prefbutton", btn);
				var disBtn = false;
				// Loop through elements, compare type and value
				for(var i in depArr)
					if((depArr[i].localName == "textbox" && JSWOA_core.strTrim(depArr[i].value) == "") || (depArr[i].localName == "radiogroup" && depArr[i].selectedIndex == -1) || (depArr[i].localName == "tree" && depArr[i].currentIndex == -1))
						disBtn = true;
				document.getElementById(btn).disabled = disBtn;
			}
		} catch(e) { alert("JSWOA_pref.setButton()\n\n" + e); }
	},
			

	/* SETVERSIONPREFS()
	 * Enable/disable version specific prefs
	 */
	setVersionPrefs: function(thisVer) {
		try {
			// Loop through JSW:O v3.x only elements
			var verArr = document.getElementsByAttribute("prefversion", "3.x");
			for(var i in verArr) {
				try {
					verArr[i].disabled = (thisVer == "3.x") ? false : true;
				} catch(e) { }
			}
		} catch(e) { alert("JSWOA_pref.setVersionPrefs()\n\n" + e); }
	},


	/* TREEADD()
	 * Add a child to the tree
	 */
	treeAdd: function(treeid, btnid, cellid, reset) {
		try {
			if(treeid) {
				if(cellid.length != 0) {
					var tValArr = [];
					// Loop through the elements for each cell
					for(var i in cellid) {
						var tEl = document.getElementById(cellid[i]);
						// Get the value and add to the array
						tValArr.push(JSWOA_core.strTrim(tEl.value));
						if(reset.length >= i)
							tEl.value = reset[i];
					}
					// Get the tree children
					var tree = document.getElementById(treeid);
					var treeList = tree.getElementsByTagName("treechildren")[0];
					var tAddNew = true;
					// Loop through tree children to see if item already exists
					if(treeList.childNodes.length != 0) {
						for(i in treeList.childNodes) {
							// Loop through each cell in the child
							for(var j in treeList.childNodes[i].childNodes) {
								var tRow = treeList.childNodes[i].childNodes[j];
								try {
									// If the first cell matches, update all the others with the new values
									if(tRow.childNodes[0].getAttribute("label").toLowerCase() == tValArr[0].toLowerCase()) {
										for(var k in tValArr) {
											if(k != 0)
												tRow.childNodes[k].setAttribute("label", tValArr[k]);
										}
										tAddNew = false;
									}
								} catch (e) {}
							}
						}
					}
					if(tAddNew)
						// Add row as new (rather than edit) 
						treeList.appendChild(JSWOA_pref.treeNewRow(tValArr));
					if(btnid)
						// Disable a button (if specified)
						document.getElementById(btnid).disabled = true;
				}
			}
		} catch(e) { alert("JSWOA_pref.treeAdd()\n\n" + e); }
	},


	/* TREEGET()
	 * Return the contents of a tree as a list
	 */
	treeGet: function(tree) {
		try {
			var tStr = "";
			var tItems = tree.getElementsByTagName("treechildren")[0].childNodes;
			if(tItems) {
				// Loop through the tree items
				for(var i in tItems) {
					for(var j in tItems[i].childNodes) {
						var tRow = tItems[i].childNodes[j];
						// Loop through the row cells
						for(var k in tRow.childNodes) {
							try {
								// Add to the list
								tStr = tStr + ((k == 0 && tStr.length != 0) ? ";" : ((k != 0) ? "," : "")) + tRow.childNodes[k].getAttribute("label");
							} catch(e) {}
						}
					}
				}
			}
			return tStr;
		} catch(e) { alert("JSWOA_pref.treeGet()\n\n" + e); }
	},


	/* TREEJL()
	 * Add columns to the tree
	 */
	treeJL: function(tree, val, tolab) {
		try {
			if(tree.id == "jswoa_pref_jltree") {
				var vLst = ["join", "leave", "both"];
				// Loop through the columns
				for(var i in vLst) {
					val = val.replace(new RegExp(("," + (tolab ? vLst[i] : JSWOA_core.strBun.getString("jsw.prf." + vLst[i]))), "gi"), ("," + (tolab ? JSWOA_core.strBun.getString("jsw.prf." + vLst[i]) : vLst[i])));
				}
			}
			return val;
		} catch(e) { alert("JSWOA_pref.treeJL()\n\n" + e); }
	},


	/* TREENEWROW()
	 * Create a new row to go in a tree
	 */
	treeNewRow: function(cells) {
		try {
			// Create the new item and row
			var newItem = document.createElement("treeitem");
			var newRow = document.createElement("treerow");
			newItem.appendChild(newRow);
			// Add cells
			for(var i in cells) {
				var newCell = document.createElement("treecell");
				newCell.setAttribute("label", cells[i]);
				newRow.appendChild(newCell);
			}
			// Pass it back
			return newItem;
		} catch(e) { alert("JSWOA_pref.treeNewRow()\n\n" + e); }
	},


	/* TREEREMOVE()
	 * Remove the selected child from a tree
	 */
	treeRemove: function(treeid, btnid) {
		try {
			if(treeid) {
				// Get the tree and selected item
				var tree = document.getElementById(treeid);
				if(tree.currentIndex != -1) {
					var treeList = tree.getElementsByTagName("treechildren")[0];
					// Remove selected item
					if(treeList)
						treeList.removeChild(treeList.childNodes.item(tree.currentIndex));
					// De-select tree
					tree.currentIndex = -1;
					// Disable the specified button
					if(btnid)
						document.getElementById(btnid).disabled = true;
				}
			}
		} catch(e) { alert("JSWOA_pref.treeRemove()\n\n" + e); }
	},


	/* TREESET()
	 * Populate a tree with the contents of a list
	 */
	treeSet: function(tree, list) {
		try {
			if(list.length != 0) {
				var tCols = tree.childNodes.length;
				// Get the tree children
				var tItems = tree.getElementsByTagName("treechildren")[0];
				if(tItems) {
					var tRows = list.split(";");
					// Loop through the list items
					for(var i in tRows) {
						var tRow = tRows[i].split(",");
						// Add the item as a row
						if(tRow.length == tCols && tRow[0] != "")
							tItems.appendChild(JSWOA_pref.treeNewRow(tRow));
					}
				}
			}
		} catch(e) { alert("JSWOA_pref.treeSet()\n\n" + e); }
	}

};
window.addEventListener("load", function() { JSWOA_pref.init(); }, false);
