/* **********************************************************************
 *  JSW:O ALERT
 *  Version: 1.4.3 (04-Sep-2011)
 *  Author: Kris King (aka Fragile)
 *
 *  See licence.txt and readme.txt for more information
 *
 *  Send bug reports, suggestions, and comments to:
 *  entrancetohades@distant-angel.co.uk
 *
 *  UI.JS
 *  Handle almost all of the major functionality. Initialise display,
 *  retrieve info from the server, pull it apart, and display it.
 * ********************************************************************** */

var JSWOA_ui = {

	// Set some variables
	fetchTimer: null, // Instance of nsITimer for server fetch
	fetchTimerOn: false, // Is timer running?
	justStarted: true, // Has application just started?
	lastHTTPErr: false, // Did we have a network error last time?
	lastHTTPReq: null, // Store last HTTP request
	lastSBText: null, // Store last status bar text
	lastSvrArr: null, // Store last server array
	contactSvr: false, // Are we contacting the server?
	netErr: false, // Did we have a network error?
	flashTimer: null, // Instance of nsITimer for Willy flash
	flashTimerOn: false, // Is flash timer running? 
	flashState: 1, // Current image opacity
	tipTimer: null, // Instance of nsITimer for closing tooltip
	tipTimerOn: false, // Is tip timer running?

	// Server list URLs for JSW:O v2.19 and v3.x
	lobbyURLV2: "http://jsw.ovine.net/getLobby2.php",
	lobbyURLV3: "http://jsw.ovine.net/getLobby3.php",


	/* AFTERSAVEPREF()
	 * Called when prefs have changed, forces update of info
	 */
	afterSavePref: function(doCon) {
		try {
			// Force a re-fetch of the server list
			JSWOA_ui.populateTooltip(JSWOA_ui.lastSvrArr);
			if(doCon) {
				JSWOA_ui.lastHTTPReq = JSWOA_ui.lastHTTPReq + "@";
				JSWOA_ui.startTimer(250);
			} else {
				// Set status text
				JSWOA_ui.lastSBText = JSWOA_core.formatStatus(JSWOA_ui.netErr ? -1 : JSWOA_ui.lastSvrArr.length);
				JSWOA_ui.setStatus(JSWOA_ui.lastSBText, false, JSWOA_ui.netErr);
			}
		} catch(e) { alert("JSWOA_ui.afterSavePref()\n\n" + e); }
	},


	/* ALARM()
	 * Called on timeout to perform update
	 */
	alarm: function(event) {
		try {
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				var oldWin = winEnum.getNext();
				oldWin.JSWOA_ui.update(event, false);
				oldWin.JSWOA_ui.startTimer();
			}
		} catch(e) { alert("JSWOA_ui.alarm()\n\n" + e); }
	},


	/* ALARMFLASH()
	 * Called on timeout to flash Willy
	 */
	alarmFlash: function(event) {
		try {
			JSWOA_ui.flashWilly(event, false);
			JSWOA_ui.startFlashTimer();
		} catch(e) { alert("JSWOA_ui.alarmFlash()\n\n" + e); }
	},


	/* EXTRACTPLAYERS()
	 * Convert player list to array, filtering out unwanted items and duplicates
	 */
	extractPlayers: function(tList) {
		try {
			// Set some variables
			var sUser = "server";
			var tArr = tList.split(",");
			var tNewArr = [];
			var tHash = new Object();
			// Remove blanks and "server" (depending on undocumented pref)
			for(i in tArr)
				if(tArr[i].length != 0 && ((tArr[i].toLowerCase() == sUser && !JSWOA_core.getPref("bool", "ignoreserverplayer")) || tArr[i].toLowerCase() != sUser))
					tHash[tArr[i]] = true;
			// Build and return array
			for(i in tHash)
				tNewArr.push(i);
			return tNewArr;
		} catch(e) { alert("JSWOA_ui.extractPlayers()\n\n" + e); }
	},


	/* FLASHWILLY()
	 * Flash the status bar Willy icon
	 */
	flashWilly: function(event, imm) {
		try {
			// Loop through all windows
			var winEnum = JSWOA_core.getWindowEnumerator();
			JSWOA_ui.flashState = (JSWOA_ui.flashState == 1) ? 0.2 : 1;
			if(typeof winEnum != "undefined") {
				while(winEnum.hasMoreElements()) {
					var tDoc = winEnum.getNext().document;
					// Change image style
					var tImg = tDoc.getElementById("jswoa_sbar_image");
					if(tImg != null)
						tImg.setAttribute("style", "opacity: " + JSWOA_ui.flashState + ";");
				}
			}
		} catch(e) { alert("JSWOA_ui.flashWilly()\n\n" + e); }
	},


	/* HIDETOOLTIP()
	 * Hide the tooltip
	 */
	hideTooltip: function(imm) {
		try {
			if(typeof imm != "undefined" && imm) {
				// Hide the tooltip
				JSWOA_ui.tipTimerOn = false;
				document.getElementById("jswoa_tooltip").hidePopup();
			} else {
				// Set timer to come back and hide the tooltip
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_ui.hideTooltip(true); } };
				JSWOA_ui.tipTimer.init(timeEvt, 250, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
				JSWOA_ui.tipTimerOn = true;
			}
		} catch(e) { alert("JSWOA_ui.hideTooltip()\n\n" + e); }
	},


	/* INIT()
	 * Called when an application window opens (main entry point)
	 */
	init: function(event) {
		try {
			// Get instances of timer service for fetch, flash, and tooltip close
			JSWOA_ui.fetchTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
			JSWOA_ui.flashTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
			JSWOA_ui.tipTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
			// Get first window opened (as that's where we're running)
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				var oldWin = winEnum.getNext();
				// If we're the oldest window, set up the status text
				if(window == oldWin) {
					var tLbl = document.getElementById("jswoa_sbar_label");
					if(tLbl != null)
						tLbl.setAttribute("value", JSWOA_core.formatStatus(0));
				} else
					// If not, tell the oldest window to update us
					oldWin.JSWOA_ui.reInit();
				// Kick everything off
				JSWOA_ui.alarm(event);
			}
		} catch(e) { alert("JSWOA_ui.init()\n\n" + e); }
	},


	/* LAUNCHJSWO()
	 * Launch JSW:O application
	 */
	launchJSWO: function(ip, port) {
		try {
			// Get prefs for JSW:O application, args etc.
			var prfJSWOApplication = JSWOA_core.getPref("char", "jswoapp");
			var prfJSWOArgs = JSWOA_core.getPref("char", "jswoargs");
			var prfJSWOVersion = JSWOA_core.getPref("char", "jswoversion");
			var prfJSWOLaunchDisable = JSWOA_core.getPref("bool", "jswolaunchdisable");

			// Set up local file and process
			var nsIProcess = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
			var nsILocalFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

			// Check the specified file exists
			try {
				nsILocalFile.initWithPath(JSWOA_core.strSlash(prfJSWOApplication));
				var appExists = nsILocalFile.exists();
			} catch(e) {
				// Exception (likely a dodgy file/path)
				var appExists = false;
			}

			// If the executable exists, launch it
			if(appExists) {
				var paramArr = prfJSWOArgs ? prfJSWOArgs.split(/\s+/) : [];
				// Launch specific server (JSW:O v3.x only)
				if(prfJSWOVersion == "3.x") {
					if(typeof ip != "undefined" && ip)
						paramArr.push("/ip " + ip);
					if(typeof port != "undefined" && port)
						paramArr.push("/port " + port);
				}
				// Disable popups (if specified)
				if(prfJSWOLaunchDisable)
					JSWOA_ui.switchPopups(true);
				// Mac needs to launch using open
				if(JSWOA_core.osPlatform == "darwin") {
					paramArr.unshift("-a", nsILocalFile.path);
					nsILocalFile.followLinks = true;
					nsILocalFile.initWithPath("/usr/bin/open");
				}
				// Launch JSW:O application (with args)
				nsIProcess.init(nsILocalFile);
				nsIProcess.run(false, paramArr, paramArr.length);
			} else {
				// JSW:O executable doesn't exist
				JSWOA_core.message(window, JSWOA_core.strBun.getString("jsw.err.jswoalert"), "error", JSWOA_core.strBun.getString("jsw.err.nojswoexe1"), JSWOA_core.strBun.getString("jsw.err.nojswoexe2"));
			}
		} catch(e) { alert("JSWOA_ui.launchJSWO()\n\n" + e); }
	},


	/* OPENPREFS()
	 * Open preferences dialog
	 */
	openPrefs: function() {
		try {
			var winEnum = JSWOA_core.getWindowEnumerator("mozilla:jswoaprefs");
			if(typeof winEnum != "undefined" && winEnum && winEnum.hasMoreElements())
				winEnum.getNext().focus();
			else
				window.openDialog("chrome://jswoa/content/preferences/preferences.xul", "jswoa_pref_dialog", "centerscreen,chrome,close,dialog=no");
		} catch(e) { alert("JSWOA_ui.openPrefs()\n\n" + e); }
	},


	/* PLAYERINARRAY()
	 * Return server array element containing player (or false if not found)
	 */
	playerInArray: function(arr, ply) {
		try {
			var tFound = false;
			var tPly = ply.toLowerCase();
			for(i in arr) {
				var tPlayArr = arr[i]["Plyr"];
				// Loop through array
				for(j in tPlayArr) {
					if(tPlayArr[j].toLowerCase() == tPly) {
						var tRet = [];
						var tKeyArr = ["IP","Desc","Type","Plyr","Pass","Cnty","Port"];
						// Copy elements to return array
						for(k in tKeyArr)
							tRet[tKeyArr[k]] = arr[i][tKeyArr[k]];
						tFound = true;
						break;
					}
				}
			}
			return tFound ? tRet : false;
		} catch(e) { alert("JSWOA_ui.playerInArray()\n\n" + e); }
	},


	/* POPULATETOOLTIP()
	 * Fill tooltip with details of running servers
	 */
	populateTooltip: function(msgArr) {
		try {
			// Loop through all open windows
			var winEnum = JSWOA_core.getWindowEnumerator();
			while(winEnum.hasMoreElements()) {
				var tDoc = winEnum.getNext().document;
				if(tDoc != null) {

					// Set some variables
					var tVBox = tDoc.getElementById("jswoa_tooltip_serverlist");
					var tSBPopup = tDoc.getElementById("jswoa_sbar_popup");
					if(tSBPopup) {
						var tSBSep = tDoc.getElementById("jswoa_sbar_llsep");
						var tMItem = tSBPopup.firstChild;
						var tHBox;
						var tTBox;

						// Get colour and version/flag prefs
						var prfText = JSWOA_core.getPref("char", "tooltiptext");
						var prfPlayers = JSWOA_core.getPref("char", "tooltipplayerlist");
						var prfShowList = JSWOA_core.getPref("bool", "tooltipshowlist");
						var prfJSWOVersion = JSWOA_core.getPref("char", "jswoversion");

						// Clear all but the first hBox
						var tTmpBox = tVBox.firstChild.nextSibling;
						while(tTmpBox.nextSibling) {
							tVBox.removeChild(tTmpBox.nextSibling);
						}

						// Clear all but the first launch menu item (JSW:O v3.x only)
						var tTmpMItem = tMItem;
						tTmpMItem.setAttribute("hidden", (prfJSWOVersion != "3.x"));
						tTmpMItem.setAttribute("image", "");
						tTmpMItem.nextSibling.setAttribute("hidden", (prfJSWOVersion != "3.x"));

						// Loop through menu items
						while(tTmpMItem.nextSibling) {
							if(tTmpMItem.nextSibling.getAttribute("menutype") == "launch")
								tSBPopup.removeChild(tTmpMItem.nextSibling);
							else
								tTmpMItem = tTmpMItem.nextSibling;
						}

						// Set title colour
						tVBox.firstChild.setAttribute("style", "color: " + JSWOA_core.getPref("char", "tooltiptitle") + ";");

						// Loop through messages
						if(typeof msgArr != "undefined" && msgArr != null && msgArr.length) {
							for(i in msgArr) {
								// First message uses existing hBox, others create a clone of it
								tHBox = (i == 0) ? tVBox.firstChild.nextSibling : tVBox.firstChild.nextSibling.cloneNode(true);
								tHBox.firstChild.setAttribute("hidden", true);
								// Set country flag (JSW:O v3.x only)
								if(prfJSWOVersion == "3.x" && JSWOA_core.getPref("bool", "tooltipflag") && msgArr[i]["Cnty"] != "") {
									var coPos = JSWOA_core.getFlagPos(msgArr[i]["Cnty"]);
									if(coPos != "") {
										tHBox.firstChild.setAttribute("hidden", false);
										tHBox.firstChild.setAttribute("style", "-moz-image-region: rect(" + coPos + ");");
									}
								}
								// Description label
								tTBox = tHBox.firstChild.nextSibling;
								tTBox.firstChild.setAttribute("hidden", false);
								tTBox.firstChild.setAttribute("value", msgArr[i]["Desc"]);
								tTBox.firstChild.setAttribute("style", "color: " + prfText + ";");
								// Type label
								tTBox.firstChild.nextSibling.setAttribute("hidden", false);
								tTBox.firstChild.nextSibling.setAttribute("value", msgArr[i]["Type"]);
								tTBox.firstChild.nextSibling.setAttribute("style", "color: " + prfText + ";");
								// Player list label
								tTBox.firstChild.nextSibling.nextSibling.setAttribute("hidden", !prfShowList);
								if(prfShowList) {
									tTBox.firstChild.nextSibling.nextSibling.setAttribute("value", (msgArr[i]["Plyr"].length ? msgArr[i]["Plyr"] : JSWOA_core.strBun.getString("jsw.val.noplayers")));
									tTBox.firstChild.nextSibling.nextSibling.setAttribute("style", "color: " + prfPlayers + ";");
								}
								// If game is not passworded, don't display lock icon
								tTBox.nextSibling.firstChild.setAttribute("hidden", (msgArr[i]["Pass"] == "N") ? true : false);
								// Append new box (if not the first message)
								if(i != 0)
									tVBox.appendChild(tHBox);
								// Add server to launch menu (JSW:O v3.x only)
								if(prfJSWOVersion == "3.x") {
									// First message uses existing menu item, others create a clone of it
									tTmpMItem = (i == 0) ? tMItem : tMItem.cloneNode(true);
									tTmpMItem.setAttribute("label", msgArr[i]["Desc"]);
									tTmpMItem.setAttribute("disabled", false);
									tTmpMItem.addEventListener("command", function () { JSWOA_ui.launchJSWO(msgArr[i]["IP"], msgArr[i]["Port"]); }, false);
									// If game is passworded, display lock icon
									if(msgArr[i]["Pass"] != "N")
										tTmpMItem.setAttribute("image", "chrome://jswoa/skin/ui/lock_small.gif");
									// Insert new menu item
									if(i != 0)
										tSBPopup.insertBefore(tTmpMItem, tSBSep);
								}
							}
						}

						// No servers
						else {
							tTBox = tVBox.firstChild.nextSibling.firstChild.nextSibling;
							// Set tooltip labels
							tTBox.firstChild.setAttribute("hidden", true);
							tTBox.firstChild.nextSibling.setAttribute("hidden", true);
							tTBox.firstChild.nextSibling.nextSibling.setAttribute("value", JSWOA_core.strBun.getString("jsw.lbl.noservers"));
							tTBox.firstChild.nextSibling.nextSibling.setAttribute("style", "color: " + prfPlayers + ";");
							tTBox.nextSibling.firstChild.setAttribute("hidden", true);
							// Set first launch menu item label (JSW:O v3.x only)
							if(prfJSWOVersion == "3.x") {
								tMItem.setAttribute("label", JSWOA_core.strBun.getString("jsw.lbl.noservers"));
								tMItem.setAttribute("disabled", true);
							}
						}
					}
				}
			}
		} catch(e) { alert("JSWOA_ui.populateTooltip()\n\n" + e); }
	},


	/* REINIT()
	 * Called when window opens that isn't the oldest to keep the status bar and tooltip in-sync
	 */
	reInit: function(error) {
		try {
			if(JSWOA_core.strBun != null) {
				if(typeof error != "undefined" && error) {
					JSWOA_ui.netErr = error;
					JSWOA_ui.lastSBText = JSWOA_core.formatStatus(-1);
				}
				JSWOA_ui.populateTooltip(JSWOA_ui.lastSvrArr);
				JSWOA_ui.setStatus(JSWOA_ui.lastSBText, false, JSWOA_ui.netErr);
			}
		} catch(e) { alert("JSWOA_ui.reInit()\n\n" + e); }
	},


	/* SETSTATUS()
	 * Set status bar text/icon
	 */
	setStatus: function(sbtext, walk, error) {
		try {
			// Set some variables
			JSWOA_ui.contactSvr = walk;
			var prfAlertsDisabled = JSWOA_core.getPref("bool", "alertsdisabled");
			// Loop through all open windows
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined") {
				while(winEnum.hasMoreElements()) {
					var tDoc = winEnum.getNext().document;
					// Set status bar text and alerts disabled status
					if(tDoc != null) {
						var tLbl = tDoc.getElementById("jswoa_sbar_label");
						if(tLbl != null) {
							if(sbtext != null)
								tLbl.setAttribute("value", sbtext);
							tLbl.disabled = (walk ? true : prfAlertsDisabled);
							tLbl.setAttribute("style", ("cursor: " + (walk ? "wait" : "default") + ";"));
							// Show label (hidden on init)
							if(!walk)
								tLbl.setAttribute("hidden", false);
						}
						// Set Willy icon image, tooltip, and cursor
						var tImg = tDoc.getElementById("jswoa_sbar_image");
						if(tImg != null) {
							var tClickAct = JSWOA_core.getPref("char", "willyclick");
							var tClickRightAct = JSWOA_core.getPref("char", "willyclickright");
							// Show disabled image and/or walking animation (if necessary)
							tImg.src = ("chrome://jswoa/skin/ui/" + (error ? "err" : (walk ? "walk" : "icon") + "10x18" + (prfAlertsDisabled ? "_dis" : "")) + ".gif");
							tImg.setAttribute("style", ("cursor: " + (walk ? "wait" : ((tClickAct == "ignore" && tClickRightAct == "ignore") ? "default" : "pointer")) + "; opacity: 1;"));
							if(tClickAct != "ignore" || tClickRightAct != "ignore") {
								// Set tooltip text for the left mouse function
								tTip = tDoc.getElementById("jswoa_willytip_lbl1");
								tTip.setAttribute("value", (tClickAct == "ignore") ? "" : (JSWOA_core.strBun.getString("jsw.lbl.click") + " " + JSWOA_core.strBun.getString("jsw.lbl.click" + ((tClickAct == "menu") ? "menu" : (prfAlertsDisabled ? "enable" : "disable")))));
								tTip.setAttribute("hidden", tClickAct == "ignore");
								// Set tooltip text for the right/cmd mouse function
								tTip = tDoc.getElementById("jswoa_willytip_lbl2");
								tTip.setAttribute("value", (tClickRightAct == "ignore") ? "" : (JSWOA_core.strBun.getString("jsw.lbl.clickright") + " " + JSWOA_core.strBun.getString("jsw.lbl.click" + ((tClickRightAct == "menu") ? "menu" : (prfAlertsDisabled ? "enable" : "disable")))));
								tTip.setAttribute("hidden", tClickRightAct == "ignore");
								tImg.setAttribute("tooltip", "jswoa_willytip");
							} else
								// Both left/right set to "ignore"; get rid of tooltip
								tImg.setAttribute("tooltip", "_child");
						}
						// Set tooltip opacity (if network error occurred)
						var tTip = tDoc.getElementById("jswoa_tooltip_serverlist");
						if(tTip != null)
							tTip.setAttribute("style", "opacity: " + (error ? 0.2 : 1) + ";");
					}
				}
			}
			JSWOA_ui.netErr = error;
		} catch(e) { alert("JSWOA_ui.setStatus()\n\n" + e); }
	},


	/* SHOWALERT()
	 * Show alert popup(s)
	 */
	showAlert: function(msgArr) {
		try {
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				// Only show alert for the oldest window
				var oldWin = winEnum.getNext();
				if(window == oldWin) {
					// Hide tooltip and open alert
					JSWOA_ui.hideTooltip(true);
					window.openDialog("chrome://jswoa/content/alert/alert.xul", "alert:alert", "chrome,dialog=yes,titlebar=no,popup=yes", msgArr);
				}
			}
		} catch(e) { alert("JSWOA_ui.showAlert()\n\n" + e); }
	},


	/* SHOWTOOLTIP()
	 * Show servers tooltip
	 */
	showTooltip: function() {
		try {
			// Only if pref set and we're not contacting the server
			if(!JSWOA_ui.contactSvr && JSWOA_core.getPref("bool", "tooltipshow")) {
				if(JSWOA_ui.tipTimerOn) {
					// Cancel the tooltip close timer
					JSWOA_ui.tipTimer.cancel();
					JSWOA_ui.tipTimerOn = false;
				}
				var ttAnchor = document.getElementById("jswoa_sbar_panel");
				var ttPopup = document.getElementById("jswoa_tooltip");
				// Set tooltip colour and open
				document.getElementById("jswoa_tooltip_colour").setAttribute("style", ("background-color: " + JSWOA_core.getPref("char", "tooltipback") + ";" + (!JSWOA_core.getPref("bool", "tooltipgradient") ? " background-image: url(none);" : "")));
				document.getElementById("jswoa_sbar_popup").hidePopup();
				JSWOA_core.openPopup(ttPopup, ttAnchor);
			}
		} catch(e) { alert("JSWOA_ui.showTooltip()\n\n" + e); }
	},


	/* STARTFLASHTIMER()
	 * Start "Willy Flash" timer
	 */
	startFlashTimer: function() {
		try {
			JSWOA_ui.stopFlashTimer();
			var timeEvt = { observe: function(subject, topic, data) { JSWOA_ui.alarmFlash(); } };
			JSWOA_ui.flashTimer.init(timeEvt, 500, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
			JSWOA_ui.flashTimerOn = true;
		} catch(e) { alert("JSWOA_ui.startFlashTimer()\n\n" + e); }
	},


	/* STARTTIMER()
	 * Start window timer
	 */
	startTimer: function(dur) {
		try {
			var fetchFreq = (typeof dur != 'undefined' && !isNaN(dur)) ? dur : (JSWOA_core.getPref("int", "fetchfreq") * 60000);
			JSWOA_ui.stopTimer();
			if(fetchFreq != 0) {
				var timeEvt = { observe: function(subject, topic, data) { JSWOA_ui.alarm(); } };
				JSWOA_ui.fetchTimer.init(timeEvt, fetchFreq, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
				JSWOA_ui.fetchTimerOn = true;
			}
		} catch(e) { alert("JSWOA_ui.startTimer()\n\n" + e); }
	},


	/* STATUSCLICK()
	 * Respond to status bar mouse clicks
	 */
	statusClick: function(event) {
		try {
			// Get the first window opened
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				var oldWin = winEnum.getNext();
				// Stop flashing Willy
				if(oldWin.JSWOA_ui.flashTimerOn) {
					oldWin.JSWOA_ui.stopFlashTimer();
					if(oldWin.JSWOA_ui.flashState != 1)
						// Return it to normal
						oldWin.JSWOA_ui.flashWilly(event, false);
				} else {
					JSWOA_ui.setStatus(JSWOA_ui.lastSBText, JSWOA_ui.contactSvr, JSWOA_ui.netErr);
					if(!JSWOA_ui.contactSvr && event.button != 1) {
						// Get action preference for the mouse button clicked
						var prfClickAct = JSWOA_core.getPref("char", "willyclick" + (event.button == 2 || (JSWOA_core.osPlatform == "darwin" && event.button == 0 && (event.metaKey || event.ctrlKey)) ? "right" : ""));
						var prfAlertsDisabled = JSWOA_core.getPref("bool", "alertsdisabled");
						// Enable/Disable
						if(prfClickAct == "switch")
							JSWOA_ui.switchPopups(!prfAlertsDisabled);
						// Show Menu
						else if(prfClickAct == "menu") {
							JSWOA_ui.hideTooltip(true);
							var wAnchor = document.getElementById("jswoa_sbar_image");
							var sbPop = document.getElementById("jswoa_sbar_popup");
							// Show/hide relevant menu options
							document.getElementById("jswoa_sbar_enable").hidden = !prfAlertsDisabled;
							document.getElementById("jswoa_sbar_disable").hidden = prfAlertsDisabled;
							// Open popup
							sbPop.hidePopup();
							JSWOA_core.openPopup(sbPop, wAnchor);
						}
					}
				}
			}
			event.preventDefault();
		} catch(e) { alert("JSWOA_ui.statusClick()\n\n" + e); }
	},


	/* STOPTIMER()
	 * Stop window timer
	 */
	stopTimer: function() {
		try {
			if(JSWOA_ui.fetchTimerOn) {
				JSWOA_ui.fetchTimer.cancel();
				JSWOA_ui.fetchTimerOn = false;
			}
		} catch(e) { alert("JSWOA_ui.stopTimer()\n\n" + e); }
	},


	/* STOPFLASHTIMER()
	 * Stop "Willy Flash" timer
	 */
	stopFlashTimer: function() {
		try {
			// Get the oldest window
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				var oldWin = winEnum.getNext();
				if(window == oldWin) {
					// Switch off the timer (if we're the oldest window)
					if(JSWOA_ui.flashTimerOn) {
						JSWOA_ui.flashTimer.cancel();
						JSWOA_ui.flashTimerOn = false;
					}
				} else
					// Tell the oldest window to do it
					oldWin.JSWOA_ui.stopFlashTimer();
			}
		} catch(e) { alert("JSWOA_ui.stopFlashTimer()\n\n" + e); }
	},


	/* SWITCHPOPUPS()
	 * Enable/disable popups
	 */
	switchPopups: function(dis) {
		try {
			JSWOA_core.setPref("bool", "alertsdisabled", dis);
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements())
				winEnum.getNext().JSWOA_ui.reInit();
		} catch(e) { alert("JSWOA_ui.switchPopups()\n\n" + e); }
	},


	/* UPDATE()
	 * Fetch JSW:O server details and alert user to any changes
	 */
	update: function(event, imm) {
		try {
			// Set some variables
			var me = this;
			var httpReq;
			var httpTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
			var httpTimerRunning = false;

			// Get prefs
			var prfAlertOnNewServer = JSWOA_core.getPref("bool", "alertonnewserver");
			var prfIgnorePwd = JSWOA_core.getPref("bool", "ignorepwd");
			var prfAlertOnShutdown = JSWOA_core.getPref("bool", "alertonshutdown");
			var prfAlertOnGameStart = JSWOA_core.getPref("bool", "alertongamestart");
			var prfAlertOnGameEnd = JSWOA_core.getPref("bool", "alertongameend");
			var prfAlertOnXPlayers = (JSWOA_core.getPref("bool", "alertonxplayerscheck")) ? JSWOA_core.getPref("int", "alertonxplayers") : 0;
			var prfAlertOnJoinLeaveList = (JSWOA_core.getPref("bool", "alertonjoinleave")) ? JSWOA_core.getPref("char", "alertonjoinleavelist") : "";
			var prfJSWOVersion = JSWOA_core.getPref("char", "jswoversion");

			// Set server list URL based on JSW:O version
			var tmpLobbyURL = (prfJSWOVersion == "3.x") ? me.lobbyURLV3 : me.lobbyURLV2;

			// STARTTIMEOUT: Start HTTP request timeout 
			me.startTimeout = function() {
				var prfHTTPReqTimeout = JSWOA_core.getPref("int", "httptimeout");
				me.stopTimeout();
				var timeEvt = { observe: function(subject, topic, data) { me.stopTimeout(); } };
				httpTimer.init(timeEvt, (prfHTTPReqTimeout * 1000), Components.interfaces.nsITimer.TYPE_ONE_SHOT);  
				httpTimerRunning = true;
			}

			// STOPTIMEOUT(): Stop HTTP request timeout 
			me.stopTimeout = function() {
				if(httpTimerRunning) {
					httpTimer.cancel();
					httpTimerRunning = false;
				}
			}

			// STATECHANGE(): Respond to HTTP request state change
			me.stateChange = function() {

				var tNewSBText = null;
				var tSvrArr = [];
				var valInLobby = "In Lobby";
				var tRow;
				var tArr;
				var j;

				// Is request complete?
				if(httpReq != null && httpReq.readyState == 4) {
					me.stopTimeout();

					// Check status is 200 (OK)
					try {
						if(httpReq.status == 200) {

							// Response hasn't changed, revert to previous details
							if(!me.lastHTTPErr && me.lastHTTPReq == httpReq.responseText)
								tNewSBText = me.lastSBText;

							// Response has changed, break down the string
							else {
								me.lastHTTPErr = false;
								me.lastHTTPReq = httpReq.responseText;
								var tResArr = me.lastHTTPReq.split("\n");

								// Process server list details
								for(i in tResArr) {
									if(tResArr[i].length) {
										tRow = tResArr[i].split("@");
										// Extract row information to build server array
										if(tRow.length >= 5) {
											if(!prfIgnorePwd || (prfIgnorePwd && tRow[4] == "N")) {
												tArr = [];
												tArr["IP"] = tRow[0];
												tArr["Desc"] = (tRow[1] != "") ? JSWOA_core.strUnslash(tRow[1]) : tArr["IP"];
												tArr["Type"] = (tRow[2].length == 0) ? valInLobby : JSWOA_core.strUnslash(tRow[2]);
												tArr["Plyr"] = me.extractPlayers(JSWOA_core.strUnslash(tRow[3]));
												tArr["Pass"] = tRow[4];
												// Country/Port (JSW:O v3.x only)
												tArr["Cnty"] = (prfJSWOVersion == "3.x" && tRow.length >= 6) ? tRow[5] : "";
												tArr["Port"] = (prfJSWOVersion == "3.x" && tRow.length >= 7) ? tRow[6] : "";
												tArr["Match"] = false;
												tSvrArr.push(tArr);
											}
										}
									}
								}

								// Set status bar text
								var ct = tSvrArr.length;
								tNewSBText = JSWOA_core.formatStatus(ct);

								// Loop through server array for things to alert on
								if(me.justStarted && JSWOA_core.getPref("bool", "displayonstart") || !me.justStarted) {
									tArr = [];
									for(i in tSvrArr) {
										var serverIsNew = true;

										// If last server array isn't empty, we have something to compare against
										if(typeof me.lastSvrArr != "undefined" && me.lastSvrArr != null) {
											for(j in me.lastSvrArr) {
												// Match server on IP address and port
												if(me.lastSvrArr[j]["IP"] == tSvrArr[i]["IP"] && me.lastSvrArr[j]["Port"] == tSvrArr[i]["Port"]) {
													serverIsNew = false;
													me.lastSvrArr[j]["Match"] = true;
													// If server has x or more players, and pref set, add to alert array
													if(prfAlertOnXPlayers != 0 && tSvrArr[i]["Plyr"].length >= prfAlertOnXPlayers) {
														tSvrArr[i]["Msg"] = (JSWOA_core.strBun.getString("jsw.msg.xplayers") + " (" + prfAlertOnXPlayers + "+ " + JSWOA_core.strBun.getString("jsw.msg.xplayersplay") + ")");
														tArr.push(tSvrArr[i]);
													}
													// If game started, and pref set, add to alert array
													if(prfAlertOnGameStart && tSvrArr[i]["Type"] != valInLobby && me.lastSvrArr[j]["Type"] != tSvrArr[i]["Type"]) {
														tSvrArr[i]["Msg"] = (JSWOA_core.strBun.getString("jsw.msg.gamestarted") + " (" + tSvrArr[i]["Plyr"].length + " " + JSWOA_core.strBun.getString((tSvrArr[i]["Plyr"].length == 1) ? "jsw.msg.xplayers1play" : "jsw.msg.xplayersplay") + ")");
														tArr.push(tSvrArr[i]);
													} else {
														// If game ended, and pref set, add to alert array
														if(prfAlertOnGameEnd && tSvrArr[i]["Type"] == valInLobby && me.lastSvrArr[j]["Type"] != valInLobby) {
															tSvrArr[i]["Msg"] = JSWOA_core.strBun.getString("jsw.msg.gameended");
															tArr.push(tSvrArr[i]);
														}
													}
													break;
												}
											}
										}

										// If server is new and pref set, add to alert array
										if(serverIsNew) {
											if(prfAlertOnNewServer) {
												tSvrArr[i]["Msg"] = JSWOA_core.strBun.getString("jsw.msg.newserver");
												tArr.push(tSvrArr[i]);
											} else {
												// If server has x or more players, and pref set, add to alert array
												if(prfAlertOnXPlayers != 0 && tSvrArr[i]["Plyr"].length >= prfAlertOnXPlayers) {
													tSvrArr[i]["Msg"] = (JSWOA_core.strBun.getString("jsw.msg.xplayersnew") + " (" + prfAlertOnXPlayers + "+ " + JSWOA_core.strBun.getString("jsw.msg.xplayersplay") + ")");
													tArr.push(tSvrArr[i]);
												}
											}
										}
									}

									// If players are on the join/leave list, add to alert array
									if(prfAlertOnJoinLeaveList.length != 0) {
										var jlArr = prfAlertOnJoinLeaveList.split(";");
										// Loop through join/leave array
										for(i in jlArr) {
											var jlRow = jlArr[i].split(",");
											// See if player exists in old and new arrays
											var plyrInOld = (typeof me.lastSvrArr != "undefined" && me.lastSvrArr != null) ? me.playerInArray(me.lastSvrArr, jlRow[0]) : false;
											var plyrInNew = me.playerInArray(tSvrArr, jlRow[0]);
											// Player Joined
											if((jlRow[1] == "Join" || jlRow[1] == "Both") && (!plyrInOld && plyrInNew)) {
												plyrInNew["Msg"] = jlRow[0].toUpperCase() + " " + JSWOA_core.strBun.getString("jsw.msg.playerjoined");
												tArr.push(plyrInNew);
											}
											// Player Left
											if((jlRow[1] == "Leave" || jlRow[1] == "Both") && (plyrInOld && !plyrInNew)) {
												plyrInOld["Msg"] = jlRow[0].toUpperCase() + " " + JSWOA_core.strBun.getString("jsw.msg.playerleft");
												tArr.push(plyrInOld);
											}
										}
									}

									// If pref set, scan for shutdown servers and add to alert array
									if(typeof me.lastSvrArr != "undefined" && me.lastSvrArr != null && prfAlertOnShutdown) {
										for(i in me.lastSvrArr) {
											if(!me.lastSvrArr[i]["Match"] || !ct) {
												me.lastSvrArr[i]["Msg"] = JSWOA_core.strBun.getString("jsw.msg.serverdown");
												me.lastSvrArr[i]["Type"] = "";
												tArr.push(me.lastSvrArr[i]);
												// Ensure this can't be picked up again
												me.lastSvrArr[i]["IP"] = "";
											}
										}
									}

									// Any messages to show?
									if(tArr.length != 0) {
										if(!JSWOA_core.getPref("bool", "alertsdisabled"))
											// Show alert popups
											me.showAlert(tArr);
										if(JSWOA_core.getPref("bool", "willyflash") && !me.flashTimerOn)
											// Flash Willy
											me.startFlashTimer();
									}
								}

								// Store current array for later comparison
								me.justStarted = false;
								me.lastSvrArr = tSvrArr;
							}

							// Populate tooltip
							me.lastSBText = tNewSBText;
							JSWOA_ui.reInit();
						}

						// Error occurred (non-200 status code)
						else {
							me.lastHTTPErr = true;
							JSWOA_ui.reInit(me.lastHTTPErr);
						}
					}
					// Exception error occurred
					catch (e) {
						me.lastHTTPErr = true;
						JSWOA_ui.reInit(me.lastHTTPErr);
					}
				}
			}

			// Get oldest window
			var winEnum = JSWOA_core.getWindowEnumerator();
			if(typeof winEnum != "undefined" && winEnum.hasMoreElements()) {
				var oldWin = winEnum.getNext();
				// Only get the info if we're the oldest window
				if(window == oldWin) {
					try {
						// Attempt to fetch server list via HTTP
						me.setStatus(null, true, false);
						httpReq = new XMLHttpRequest();
						httpReq.open("GET", tmpLobbyURL, true);
						httpReq.onreadystatechange = me.stateChange;
						httpReq.send(null);
						me.startTimeout();
					}
					// Error occurred, end timeout
					catch (e) {
						me.stopTimeout();
					}
				}
			}
		} catch(e) { alert("JSWOA_ui.update()\n\n" + e); }
	}

};
window.addEventListener("load", function() { JSWOA_ui.init(); }, false);
